package com.link.core.basic.dao;

import com.link.core.basic.service.snowflake.IdWorkerSingleton;
import com.link.core.util.CoreYmlConstants;
import com.link.core.util.RedisRowIdUtil;
import com.link.core.util.RedisUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.redisclient.LinkRedisClient;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import javax.annotation.Resource;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

@Service
public class KeyGenerateDao {
   private static final Logger logger = LogManager.getLogger(KeyGenerateDao.class);
   private static final String MYSQL_ROWID_REDISKEY = "MYSQL_ROWID_CACHE";
   private static final String ORACLE_ROWID_REDISKEY = "ORACLE_ROWID_CACHE";
   @Resource
   private JdbcTemplate jdbcTemplate;

   public String keyGenerateOld() {
      String id = null;
      String sql = "";
      if (StringUtils.equals(CoreYmlConstants.getApplicationDatabase(), "mysql")) {
         sql = "select s_sequence_pkg_get_next_rowid() as result";
      } else if (StringUtils.equals(CoreYmlConstants.getApplicationDatabase(), "oracle")) {
         sql = "select s_sequence_pkg.get_next_rowid() as result from dual";
      }

      id = (String)this.jdbcTemplate.queryForObject(sql, String.class);
      logger.debug("生成的Sequence = [" + id + "]");
      return id;
   }

   public String orderNoGenerate() {
      return String.valueOf(this.keyGenerate());
   }

   public String memCardNoGenerate() {
      return String.valueOf(this.keyGenerate());
   }

   public synchronized String keyGenerateOld1() {
      LinkRedisClient jedis = null;
      String id = null;

      String var4;
      try {
         jedis = RedisUtil.getJedis();
         long rowIdNum = 0L;
         if (StringUtils.equals(CoreYmlConstants.getApplicationDatabase(), "mysql")) {
            rowIdNum = RedisRowIdUtil.redisRowIdNumBySet(jedis, "MYSQL_ROWID_CACHE");
         } else if (StringUtils.equals(CoreYmlConstants.getApplicationDatabase(), "oracle")) {
            rowIdNum = RedisRowIdUtil.redisRowIdNumByList(jedis, "ORACLE_ROWID_CACHE", 0L, -1L);
         }

         if (rowIdNum == 0L && StringUtils.equals(CoreYmlConstants.getApplicationDatabase(), "mysql")) {
            RedisRowIdUtil.generateRedisRowIdBySet(this.jdbcTemplate, jedis, 2600, "主线程");
         }

         ExecutorService threadExecutor = new ThreadPoolExecutor(5, 100, 0L, TimeUnit.MILLISECONDS, new LinkedBlockingQueue(1024));
         threadExecutor.execute(new PushRowId2RedisThreadDao(this.jdbcTemplate));
         threadExecutor.shutdown();
         if (StringUtils.equals(CoreYmlConstants.getApplicationDatabase(), "mysql")) {
            id = RedisRowIdUtil.spopUncloseJedis(jedis, "MYSQL_ROWID_CACHE");
         } else if (StringUtils.equals(CoreYmlConstants.getApplicationDatabase(), "oracle")) {
            id = RedisRowIdUtil.rpopUncloseJedis(jedis, "ORACLE_ROWID_CACHE");
            return id;
         }

         return id;
      } catch (Exception var9) {
         var4 = id;
      } finally {
         RedisUtil.returnResource(jedis);
      }

      return var4;
   }

   public String keyGenerate() {
      IdWorkerSingleton idWorker = IdWorkerSingleton.getInstance(CoreYmlConstants.getDataCenterId(), CoreYmlConstants.getMachineId());
      return String.valueOf(idWorker.nextId());
   }
}
